package jess;

import java.io.*;

class Format { 
    public Format(String s) throws JessException { 
        width = 0;
        precision = -1;
        pre = "";
        post = "";
        leadingZeroes = false;
        showPlus = false;
        alternate = false;
        showSpace = false;
        leftAlign = false;
        fmt = ' '; 
      
        int length = s.length();
        int parseState = 0; 
        // 0 = prefix, 1 = flags, 2 = width, 3 = precision,
        // 4 = format, 5 = end
        int i = 0;
      
        while (parseState == 0) { 
            if (i >= length) parseState = 5;
            else if (s.charAt(i) == '%') {
                if (i < length - 1) {
                    if (s.charAt(i + 1) == '%') { 
                        pre = pre + '%';
                        i++;
                    }
                    else
                        parseState = 1;
                }
                else throw new JessException("Format.<init>", "Invalid format", s);
            }
            else
                pre = pre + s.charAt(i);
            i++;
        }
        while (parseState == 1) {
            if (i >= length) parseState = 5;
            else if (s.charAt(i) == ' ') showSpace = true;
            else if (s.charAt(i) == '-') leftAlign = true; 
            else if (s.charAt(i) == '+') showPlus = true;
            else if (s.charAt(i) == '0') leadingZeroes = true;
            else if (s.charAt(i) == '#') alternate = true;
            else { parseState = 2; i--; }
            i++;
        }      
        while (parseState == 2) { 
            if (i >= length) parseState = 5;
            else if ('0' <= s.charAt(i) && s.charAt(i) <= '9') { 
                width = width * 10 + s.charAt(i) - '0';
                i++;
            }
            else if (s.charAt(i) == '.') { 
                parseState = 3;
                precision = 0;
                i++;
            }
            else 
                parseState = 4;            
        }
        while (parseState == 3) {
            if (i >= length) parseState = 5;
            else if ('0' <= s.charAt(i) && s.charAt(i) <= '9') {
                precision = precision * 10 + s.charAt(i) - '0';
                i++;
            }
            else 
                parseState = 4;                  
        }
        if (parseState == 4) {
            if (i >= length) parseState = 5;
            else fmt = s.charAt(i);
            i++;
        }
        if (i < length)
            post = s.substring(i, length);
    }      

    /** 
     * Formats a double into a string (like sprintf in C)
     * @param x the number to format
     * @return the formatted string 
     * @exception IllegalArgumentException if bad argument
     */
   
    public String form(double x) {  
        String r;
        if (precision < 0) precision = 6;
        int s = 1;
        if (x < 0) { x = -x; s = -1; }
        if (fmt == 'f')
            r = fixedFormat(x);
        else if (fmt == 'e' || fmt == 'E' || fmt == 'g' || fmt == 'G')
            r = expFormat(x);
        else throw new IllegalArgumentException();
        
        return pad(sign(s, r));
    }
    
    /** 
     * Formats a long integer into a string (like sprintf in C)
     * @param x the number to format
     * @return the formatted string 
     */
    
    public String form(long x) {
        String r; 
        int s = 0;
        switch (fmt) {
        case 'd':
        case 'i': 
            if (x < 0) {
                x = -x;
                s = -1;
            } else
                s = 1;           
            r = Integer.toString((int) x);
            break;
        case 'o':
            r = Integer.toString((int) x, 8);
            break;
        case 'x':
            r = Integer.toString((int) x, 16);
            break;
        case 'X':
            r = Integer.toString((int) x, 16).toUpperCase();
            break;
        default:
            throw new IllegalArgumentException();
        }
        return pad(sign(s, r));
    }
    
    /**
     * Formats a string into a larger string (like sprintf in C)
     * @param s 
     * @param x <<Invalid>> the value to format
     * @return the formatted string
     */
    
    public String form(String s) {
        if (fmt != 's')
            throw new IllegalArgumentException();
        if (precision >= 0) s = s.substring(0, precision);
        return pad(s);
    }
    
    private static String repeat(char c, int n) { 
        if (n <= 0) return "";
        StringBuffer s = new StringBuffer(n);
        for (int i = 0; i < n; i++) s.append(c);
        return s.toString();
    }
    
    private String pad(String r) { 
        String p = repeat(' ', width - r.length());
        if (leftAlign) return pre + r + p + post;
        else return pre + p + r + post;
    }

    private String sign(int s, String r) {  
        String p = "";
        if (s < 0) p = "-"; 
        else if (s > 0) {  
            if (showPlus) p = "+";
            else if (showSpace) p = " ";
        }
        else {  
            if (fmt == 'o' && alternate && r.length() > 0 && r.charAt(0) != '0') p = "0";
            else if (fmt == 'x' && alternate) p = "0x";
            else if (fmt == 'X' && alternate) p = "0X";
        }
        int w = 0;
        if (leadingZeroes) 
            w = width;
        else if ((fmt == 'd' || fmt == 'i' || fmt == 'x' || fmt == 'X' || fmt == 'o') 
                 && precision > 0) w = precision;
        
        return p + repeat('0', w - p.length() - r.length()) + r;
    }
    
    private String fixedFormat(double d) {
        String f = "";
        
        if (d > 0x7FFFFFFFFFFFFFFFL) return expFormat(d);
        
        double offset = 0;
        if (precision >= 0)
            offset = ((d > 0) ? 0.5 : -0.5) * Math.pow(10, -(precision));
        d += offset;
        long l = (long) d;
        f = f + l;

        double fr = d - l; // fractional part
        if (fr >= 1 || fr < 0) return expFormat(d);

        return f + fracPart(fr);
    }   
    
    private String fracPart(double fr) {
        String z = "";
        if (precision > 0) {  
            double factor = 1;
            String leadingZeroes = "";
            for (int i = 1;
                 i <= precision && factor <= 0x7FFFFFFFFFFFFFFFL;
                 i++) {
                factor *= 10; 
                leadingZeroes = leadingZeroes + "0"; 
            }
            long l = (long) (factor * fr);
                
            z = leadingZeroes + l;
            z = z.substring(z.length() - precision);
        }
        
        
        if (precision > 0 || alternate) z = "." + z;
        if ((fmt == 'G' || fmt == 'g') && !alternate)  { 
            int t = z.length() - 1;
            while (t >= 0 && z.charAt(t) == '0') t--;
            if (t >= 0 && z.charAt(t) == '.') t--;
            z = z.substring(0, t + 1);
        }
        return z;
    }
    
    private String expFormat(double d) {
        String f = "";
        int e = 0;
        double dd = d;
        double factor = 1;
        while (dd > 10) { e++; factor /= 10; dd = dd / 10; }
        while (dd < 1) { e--; factor *= 10; dd = dd * 10; }
        if ((fmt == 'g' || fmt == 'G') && e >= -4 && e < precision) 
            return fixedFormat(d);
        
        d = d * factor;
        f = f + fixedFormat(d);
        
        if (fmt == 'e' || fmt == 'g')
            f = f + "e";
        else
            f = f + "E";
        
        String p = "000";      
        if (e >= 0) {  
            f = f + "+";
            p = p + e;
        }
        else { 
            f = f + "-";
            p = p + (-e);
        }
        
        return f + p.substring(p.length() - 3);
    }

    public char getFormat() {
        return fmt;
    }
    
    private int width;
    private int precision;
    private String pre;
    private String post;
    private boolean leadingZeroes;
    private boolean showPlus;
    private boolean alternate;
    private boolean showSpace;
    private boolean leftAlign;
    private char fmt; // one of cdeEfgGiosxXos    
}

